@@ -1,7 +1,7 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* ibus - The Input Bus
  * Copyright (C) 2013-2014 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2013-2017 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2013-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -98,14 +98,16 @@ parse_compose_value (IBusComposeData  *compose_data,
         uch = words[1][1];
 
         /* The escaped string "\"" is separated with '\\' and '"'. */
-        if (uch == '\0' && words[2][0] == '"')
+        if (uch == '\0' && words[2][0] == '"') {
             uch = '"';
         /* The escaped octal */
-        else if (uch >= '0' && uch <= '8')
+        } else if (uch >= '0' && uch <= '8') {
             uch = g_ascii_strtoll(words[1] + 1, NULL, 8);
         /* If we need to handle other escape sequences. */
-        else if (uch != '\\')
-            g_assert_not_reached ();
+        } else if (uch != '\\') {
+            g_warning ("Invalid backslash: %s: %s", val, line);
+            goto fail;
+        }
     }
 
     if (g_utf8_get_char (g_utf8_next_char (words[1])) > 0) {
@@ -532,7 +534,7 @@ ibus_compose_table_serialize (IBusComposeTable *compose_table)
 
     variant_data = g_variant_new_fixed_array (G_VARIANT_TYPE_UINT16,
                                               compose_table->data,
-                                              index_stride * n_seqs,
+                                              (gsize)index_stride * n_seqs,
                                               sizeof (guint16));
     if (variant_data == NULL) {
         g_warning ("Could not change compose data to GVariant.");
